from alive_progress import alive_bar
from rdflib.namespace import RDF, RDFS
from abstract import Resource
from abstract.namespace import TST, SYN
from abstract.uri import trustscoreUserUri, organizationUri
from abstract.literal import floatLiteral, plainLiteral


class TrustScore(Resource):
    def __init__(self, df):
        self.__resource_df = df

    @property
    def resource_df(self):
        return self.__resource_df

    @resource_df.setter
    def resource_df(self, value):
        self.__resource_df = value

    # def convert(self, trust_score_df: pd.DataFrame, file_name: str):
    #     graph = Graph()
    #     self.__set_model(graph, self.model_path)
    #     self.__build_user_trust_graph(graph, trust_score_df)
    #     graph.serialize(
    #         destination=f"{self.destination_dir}/{file_name}", format="turtle"
    #     )

    def convert(self, graph):
        # Data custodian
        data_custodian = trustscoreUserUri("data_custodian")
        graph.add((data_custodian, RDF.type, TST.User))
        graph.add((data_custodian, RDFS.label, plainLiteral("user_data_custodian")))
        graph.add((data_custodian, TST.credibility, floatLiteral(1.0)))
        graph.add((data_custodian, TST.objectivity, floatLiteral(1.0)))
        graph.add((data_custodian, TST.trustfulness, floatLiteral(1.0)))

        rows = self.__resource_df.shape[0]
        with alive_bar(rows, force_tty=True, title="Trust score conversion") as bar:
            for index, row in self.__resource_df.iterrows():
                user = trustscoreUserUri(index)
                graph.add((user, RDF.type, TST.User))
                graph.add((user, RDFS.label, plainLiteral(f"user_{index}")))
                graph.add(
                    (
                        user,
                        TST.identityTrust,
                        floatLiteral(row["identity_trust"]),
                    )
                )
                graph.add(
                    (
                        user,
                        TST.behaviorTrust,
                        floatLiteral(row["behavior_trust"]),
                    )
                )
                graph.add(
                    (
                        user,
                        SYN.isAffiliatedWith,
                        organizationUri(row["organization"]),
                    )
                )
                graph.add(
                    (
                        organizationUri(row["organization"]),
                        SYN.hasEmployed,
                        user,
                    )
                )
                bar()

        return graph



from abc import ABC, abstractmethod
from pandas import DataFrame


class Resource(ABC):
    @abstractmethod
    def __init__(self, df: DataFrame):
        self.__resource_df = df

    @property
    @abstractmethod
    def resource_df(self):
        pass

    @resource_df.setter
    @abstractmethod
    def resource_df(self, value):
        pass

    @abstractmethod
    def convert(self, graph):
        pass



from pathlib import Path
import copy
from rdflib import Graph
import pandas as pd
from dua import resource as dua_resource
from trustscore import resource as trust_resource
from .resource import (
    Allergy,
    CarePlan,
    Claim,
    ClaimTransaction,
    Condition,
    Device,
    Encounter,
    ImagingStudy,
    Immunization,
    Medication,
    Observation,
    Organization,
    Patient,
    Payer,
    PayerTransition,
    Procedure,
    Provider,
    Supply,
)
from abstract.namespace import SYN, DUA, TST
from alive_progress import alive_bar


def split_dataframe(df: pd.DataFrame, chunk_size: int = 500000) -> list[pd.DataFrame]:
    chunks = list()
    num_chunks = len(df.index) // chunk_size + 1
    print(f"Split dataframe into {num_chunks} chunks.")
    for i in range(num_chunks):
        chunks.append(df[i * chunk_size : (i + 1) * chunk_size])
    print("Chunk sizes:")
    for c in chunks:
        print(len(c.index))
    return chunks


class GraphBuilder:
    def __init__(
        self,
        csv_dir: str,
        model_path: str,
        destination_dir: Path,
        include_dua: bool = False,
        include_trustscore: bool = False,
        skip: list[str] = None,
    ):
        self.RESOURCE_DICT = {
            "allergies.csv": self.convertAllergy,
            "careplans.csv": self.convertCarePlan,
            "claims.csv": self.convertClaim,
            "claim_transactions.csv": self.convertClaimTransaction,
            "conditions.csv": self.convertCondition,
            "devices.csv": self.convertDevice,
            "encounters.csv": self.convertEncounter,
            "imaging_studies.csv": self.convertImagingStudy,
            "immunizations.csv": self.convertImmunization,
            "medications.csv": self.convertMedication,
            "observations.csv": self.convertObservation,
            "organizations.csv": self.convertOrganization,
            "patients.csv": self.convertPatient,
            "payer_transitions.csv": self.convertPayerTransition,
            "payers.csv": self.convertPayer,
            "procedures.csv": self.convertProcedure,
            "providers.csv": self.convertProvider,
            "supplies.csv": self.convertSupply,
        }
        # Optional Trustscore and DUA
        self.include_dua = include_dua
        self.include_trustscore = include_trustscore
        self.skip = skip

        self.graph = Graph()
        self.csv_dir = Path(csv_dir)
        self.destination_dir = destination_dir
        self.setModel(model_path)

    def serialize(self, destination: str):
        return self.graph.serialize(destination=destination)

    def build(self, chunk_size: int = 500000):
        self.convert(chunk_size=chunk_size)
        return self.graph

    def convert(self, chunk_size: int = 500000):
        for file, func in self.RESOURCE_DICT.items():
            if file in self.skip:
                print(f"Skipping {file}")
                continue
            resource_path = Path(self.csv_dir / file)
            if Path(resource_path).exists():
                resource_df = pd.read_csv(resource_path)
                if len(resource_df.index) < chunk_size:
                    func(resource_df)
                    del resource_df
                else:
                    chunks = split_dataframe(resource_df, chunk_size=chunk_size)
                    i = 0
                    for chunk in chunks:
                        print(f"Converting chunk {i}")
                        func(chunk, chunk_id=i)
                        i += 1
                        del chunk
                    del resource_df

        if self.include_dua:
            dua_df = pd.read_csv(self.csv_dir / "dua.csv")
            self.convertDUA(dua_df)

            del dua_df

        if self.include_trustscore:
            trustscore_df = pd.read_csv(self.csv_dir / "trustscore.csv")
            self.convertTrustscore(trustscore_df)

            del trustscore_df

    def setModel(self, model_path):
        self.graph.parse(model_path, format="n3")
        self.graph.bind("syn", SYN)
        if self.include_dua:
            self.graph.bind("dua", DUA)
        if self.include_trustscore:
            self.graph.bind("tst", TST)
        print(f"Model has {len(self.graph)} triples.")

    def convertAllergy(self, allergy_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        allergy = Allergy(allergy_df)
        allergy.convert(graph)
        with alive_bar(
            title=f"Allergy graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"allergy_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "allergy.ttl")
            bar()

    def convertCarePlan(self, carePlan_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        careplan = CarePlan(carePlan_df)
        careplan.convert(graph)
        with alive_bar(
            title=f"Care plan graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"careplan_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "careplan.ttl")
            bar()

    def convertClaim(self, claim_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        claim = Claim(claim_df)
        claim.convert(graph)
        with alive_bar(
            title=f"Claim graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"claim_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "claim.ttl")
            bar()

    def convertClaimTransaction(self, claimTransaction_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        claimTransaction = ClaimTransaction(claimTransaction_df)
        claimTransaction.convert(graph)
        with alive_bar(
            title=f"Claim transaction graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir
                    / f"claim_transaction_{chunk_id}.ttl"
                )
            else:
                graph.serialize(
                    destination=self.destination_dir / "claim_transaction.ttl"
                )
            bar()

    def convertCondition(self, condition_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        condition = Condition(condition_df)
        condition.convert(graph)
        with alive_bar(
            title=f"Condition graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"condition_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "condition.ttl")
            bar()

    def convertDevice(self, device_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        device = Device(device_df)
        device.convert(graph)
        with alive_bar(
            title=f"Device graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"device_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "device.ttl")
            bar()

    def convertEncounter(self, encounter_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        encounter = Encounter(encounter_df)
        encounter.convert(graph)
        with alive_bar(
            title=f"Encounter graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"encounter_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "encounter.ttl")
            bar()

    def convertImagingStudy(self, imagingStudy_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        imagingStudy = ImagingStudy(imagingStudy_df)
        imagingStudy.convert(graph)
        with alive_bar(
            title=f"Imaging study graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"imaging_study_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "imaging_study.ttl")
            bar()

    def convertImmunization(self, immunization_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        immunization = Immunization(immunization_df)
        immunization.convert(graph)
        with alive_bar(
            title=f"Immunization graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"immunization_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "immunization.ttl")
            bar()

    def convertMedication(self, medication_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        medication = Medication(medication_df)
        medication.convert(graph)
        with alive_bar(
            title=f"Medication graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"medication_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "medication.ttl")
            bar()

    def convertObservation(self, observation_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        observation = Observation(observation_df)
        observation.convert(graph)
        with alive_bar(
            title=f"Observation graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"observation_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "observation.ttl")
            bar()

    def convertOrganization(self, organization_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        organization = Organization(organization_df)
        organization.convert(graph)
        with alive_bar(
            title=f"Organization graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"organization_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "organization.ttl")
            bar()

    def convertPatient(self, patient_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        patient = Patient(patient_df)
        patient.convert(graph)
        with alive_bar(
            title=f"Patient graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"patient_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "patient.ttl")
            bar()

    def convertPayer(self, payer_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        payer = Payer(payer_df)
        payer.convert(graph)
        with alive_bar(
            title=f"Payer graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"payer_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "payer.ttl")
            bar()

    def convertPayerTransition(self, payerTransition_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        payerTransition = PayerTransition(payerTransition_df)
        payerTransition.convert(graph)
        with alive_bar(
            title=f"Payer transition graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir
                    / f"payer_transition_{chunk_id}.ttl"
                )
            else:
                graph.serialize(
                    destination=self.destination_dir / "payer_transition.ttl"
                )
            bar()

    def convertProcedure(self, procedure_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        procedure = Procedure(procedure_df)
        procedure.convert(graph)
        with alive_bar(
            title=f"Procedure graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"procedure_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "procedure.ttl")
            bar()

    def convertProvider(self, provider_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        provider = Provider(provider_df)
        provider.convert(graph)
        with alive_bar(
            title=f"Provider graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"provider_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "provider.ttl")
            bar()

    def convertSupply(self, supply_df=None, chunk_id=None):
        graph = copy.deepcopy(self.graph)
        supply = Supply(supply_df)
        supply.convert(graph)
        with alive_bar(
            title=f"Supply graph {chunk_id} serialization",
            unknown="waves2",
        ) as bar:
            if chunk_id is not None:
                graph.serialize(
                    destination=self.destination_dir / f"supply_{chunk_id}.ttl"
                )
            else:
                graph.serialize(destination=self.destination_dir / "supply.ttl")
            bar()

    def convertDUA(self, dua_df=None):
        graph = copy.deepcopy(self.graph)
        dua = dua_resource.DataUsageAgreement(dua_df)
        dua.convert(graph)
        with alive_bar(
            title="DUA graph Serialization",
            unknown="waves2",
        ) as bar:
            graph.serialize(destination=self.destination_dir / "dua.ttl")
            bar()

    def convertTrustscore(self, trustscore_df=None):
        graph = copy.deepcopy(self.graph)
        trustscore = trust_resource.TrustScore(trustscore_df)
        trustscore.convert(graph)
        with alive_bar(
            title="Trust score graph Serialization",
            unknown="waves2",
        ) as bar:
            graph.serialize(destination=self.destination_dir / "trustscore.ttl")
            bar()

    # def convertCarePlan(self, carePlan_df=None):
    #     careplan = CarePlan(carePlan_df)
    #     careplan.convert(self.graph)

    # def convertClaim(self, claim_df=None):
    #     claim = Claim(claim_df)
    #     claim.convert(self.graph)

    # def convertClaimTransaction(self, claimTransaction_df=None):
    #     claimTransaction = ClaimTransaction(claimTransaction_df)
    #     claimTransaction.convert(self.graph)

    # def convertCondition(self, condition_df=None):
    #     condition = Condition(condition_df)
    #     condition.convert(self.graph)

    # def convertDevice(self, device_df=None):
    #     device = Device(device_df)
    #     device.convert(self.graph)

    # def convertEncounter(self, encounter_df=None):
    #     encounter = Encounter(encounter_df)
    #     encounter.convert(self.graph)

    # def convertImagingStudy(self, imagingStudy_df=None):
    #     imagingStudy = ImagingStudy(imagingStudy_df)
    #     imagingStudy.convert(self.graph)

    # def convertImmunization(self, immunization_df=None):
    #     immunization = Immunization(immunization_df)
    #     immunization.convert(self.graph)

    # def convertMedication(self, medication_df=None):
    #     medication = Medication(medication_df)
    #     medication.convert(self.graph)

    # def convertObservation(self, observation_df=None):
    #     observation = Observation(observation_df)
    #     observation.convert(self.graph)

    # def convertOrganization(self, organization_df=None):
    #     organization = Organization(organization_df)
    #     organization.convert(self.graph)

    # def convertPatient(self, patient_df=None):
    #     patient = Patient(patient_df)
    #     patient.convert(self.graph)

    # def convertPayer(self, payer_df=None):
    #     payer = Payer(payer_df)
    #     payer.convert(self.graph)

    # def convertPayerTransition(self, payerTransition_df=None):
    #     payerTransition = PayerTransition(payerTransition_df)
    #     payerTransition.convert(self.graph)

    # def convertProcedure(self, procedure_df=None):
    #     procedure = Procedure(procedure_df)
    #     procedure.convert(self.graph)

    # def convertProvider(self, provider_df=None):
    #     provider = Provider(provider_df)
    #     provider.convert(self.graph)

    # def convertSupply(self, supply_df=None):
    #     supply = Supply(supply_df)
    #     supply.convert(self.graph)

    # def convertDUA(self, dua_df):
    #     dua = dua_resource.DataUsageAgreement(dua_df)
    #     dua.convert(self.graph)

    # def convertTrustscore(self, trustscore_df):
    #     trust = trust_resource.TrustScore(trustscore_df)
    #     trust.convert(self.graph)



DUA_COLS = [
    "dataCustodian",
    "recipient",
    "requestedData",  # Basic Information
    "permittedUseOrDisclosure",
    "terms",
    "terminationCause",
    "terminationEffect",  # TermAndTermination
    "dataSecurityPlanAccess",
    "dataSecurityPlanProtection",
    "dataSecurityPlanStorage",
]  # Data_Security_Plan

PERMITTED_USE_OR_DISCLOSURE = [
    "treatment",
    "payment",
    "health_care_operations",
    "notification",
    "public_health",
    "limited_data_set",
]


DATA_CLASSES = [
    "Allergy",
    "CarePlan",
    "Claim",
    "ClaimTransaction",
    "Condition",
    "Device",
    "Encounter",
    "ImagingStudy",
    "Immunization",
    "Medication",
    "Observation",
    "Organization",
    "Patient",
    "PayerTransition",
    "Payer",
    "Procedure",
    "Provider",
    "Supply",
]



from alive_progress import alive_bar
from rdflib.namespace import RDF, RDFS
from abstract import Resource
from abstract.namespace import DUA, SYN
from abstract.literal import plainLiteral
from abstract.uri import (
    syntheaUri,
    duaUri,
    permittedUseOrDisclosureUri,
    dataSecurityPlanUri,
    termAndTerminationUri,
    organizationUri,
)


class DataUsageAgreement(Resource):
    def __init__(self, resource_df):
        self.__resource_df = resource_df

    @property
    def resource_df(self):
        return self.__resource_df

    @resource_df.setter
    def resource_df(self, value):
        self.__resource_df = value

    def convert(self, graph):
        """
        Should change dataCustodian and recipient values.
        It should be id instead of their names
        Synthea organization name format: SYN:organization_{organization_id}
        """
        # Data Custodian Organization
        data_custodian = organizationUri("dataCustodian")
        graph.add((data_custodian, RDF.type, SYN.Organization))
        graph.add((data_custodian, RDFS.label, plainLiteral("DataCustodian")))

        # Organizations
        rows = self.__resource_df.shape[0]
        for index, row in self.__resource_df.iterrows():
            organization = organizationUri(row["recipient"])
            graph.add((organization, RDF.type, SYN.Organization))
            graph.add((organization, RDFS.label, plainLiteral(f"Organization_{index}")))

        # Data Usage Agreement
        with alive_bar(
            rows, force_tty=True, title="Data Usage Agreement Conversion"
        ) as bar:
            for index, row in self.__resource_df.iterrows():
                dua = duaUri(index)
                data_security_plan = dataSecurityPlanUri(index)
                term_and_termination = termAndTerminationUri(index)

                graph.add((dua, RDF.type, DUA.DataUsageAgreement))
                graph.add((dua, RDFS.label, plainLiteral(f"DUA_{index}")))
                graph.add(
                    (
                        dua,
                        DUA.hasDataCustodian,
                        data_custodian,
                    )
                )
                graph.add((dua, DUA.hasRecipient, organizationUri(row["recipient"])))

                # Data Security Plan
                graph.add((data_security_plan, RDF.type, DUA.DataSecurityPlan))
                graph.add(
                    (
                        data_security_plan,
                        DUA.dataSecurityPlanAccess,
                        plainLiteral(row["dataSecurityPlanAccess"]),
                    )
                )
                graph.add(
                    (
                        data_security_plan,
                        DUA.dataSecurityPlanProtection,
                        plainLiteral(row["dataSecurityPlanProtection"]),
                    )
                )
                graph.add(
                    (
                        data_security_plan,
                        DUA.dataSecurityPlanStorage,
                        plainLiteral(row["dataSecurityPlanStorage"]),
                    )
                )
                graph.add((dua, DUA.hasDataSecurityPlan, data_security_plan))

                # Term and Termination
                graph.add((term_and_termination, RDF.type, DUA.TermAndTermination))
                graph.add((term_and_termination, DUA.terms, plainLiteral(row["terms"])))
                graph.add(
                    (
                        term_and_termination,
                        DUA.terminationCause,
                        plainLiteral(row["terminationCause"]),
                    )
                )
                graph.add(
                    (
                        term_and_termination,
                        DUA.terminationEffect,
                        plainLiteral(row["terminationEffect"]),
                    )
                )
                graph.add((dua, DUA.hasTermAndTermination, term_and_termination))

                # Permitted Use or Disclosure
                permitted_use_or_disclosure = permittedUseOrDisclosureUri(
                    row["permittedUseOrDisclosure"]
                )
                graph.add(
                    (
                        permitted_use_or_disclosure,
                        RDF.type,
                        DUA.PermittedUseOrDisclosure,
                    )
                )
                graph.add(
                    (
                        dua,
                        DUA.hasPermittedUseOrDisclosure,
                        permittedUseOrDisclosureUri(row["permittedUseOrDisclosure"]),
                    )
                )

                # Requested Data
                requested_data = row["requestedData"].split(",")
                for data in requested_data:
                    graph.add((dua, DUA.requestedData, plainLiteral(syntheaUri(data))))

                bar()



import random
import time
import pandas as pd
from pathlib import Path


class TrustScoreGenerator:
    def __init__(self, num_org: int, user_per_org: int):
        """
        Parameters
        ----------
        num_org : int
            The number of organizations.
        user_per_org : int
            The number of users per organization.
        """
        self.num_org = num_org
        self.user_per_org = user_per_org
        self.user_count = num_org * user_per_org

    def generate_trustscore(self, save_path: str):
        """
        This method generates trust score for users and organizations in the simplest way.
        It gives highest trust score to all users and organizations, which is 1.
        """
        user_id = list(range(0, self.user_count))
        user_identity_trust = [1] * self.user_count
        user_behavior_trust = [1] * self.user_count
        affiliated_org = list(range(0, self.num_org)) * self.user_per_org
        org_identity_trust = [1] * self.user_count
        data = {
            "user": user_id,
            "identity_trust": user_identity_trust,
            "behavior_trust": user_behavior_trust,
            "organization": affiliated_org,
            "organization_trust": org_identity_trust,
        }

        print("Generating trust score...")
        print(f"Number of rows of user_id: {len(user_id)}")
        print(f"Number of rows of identity_trust: {len(user_identity_trust)}")
        print(f"Number of rows of behavior_trust: {len(user_behavior_trust)}")
        print(f"Number of rows of org_identity_trust: {len(org_identity_trust)}")

        save_path = Path(save_path).resolve()
        trustscore_df = pd.DataFrame.from_dict(data)
        trustscore_df.to_csv(f"{save_path}/trustscore.csv")
        return trustscore_df


class TrustScoreGeneratorAdvanced:
    def __init__(self, num_org: int, user_per_org: int):
        self.num_org = num_org
        self.user_per_org = user_per_org
        self.user_count = num_org * user_per_org
        self.__ranges = None
        self.__identity_dist = None
        self.__behavior_dist = None
        self.__org_identity_dist = None

    def set_distribution_and_trust(
        self,
        ranges: list[tuple[float, float]],
        identity_dist: list[float],
        behavior_dist: list[float],
        org_identity_dist: list[float],
    ):
        """
        Parameters
        ----------
        ranges : list[tuple[float, float]]
            A list of tuples that contains the minimum and maximum value of the trust score.
        identity_dist : list[float]
            A list of floats that contains the distribution of the identity trust score.
        behavior_dist : list[float]
            A list of floats that contains the distribution of the behavior trust score.
        org_identity_dist : list[float]
            A list of floats that contains the distribution of the organization's identity trust score.
        """
        print(
            f"Length of ranges: {len(ranges)} identity_dist: {len(identity_dist)} behavior_dist: {len(behavior_dist)}"
        )
        if len(identity_dist) != len(ranges):
            raise ValueError(
                "The length of identity_dist must be the same as the length of ranges."
            )
        elif len(behavior_dist) != len(ranges):
            raise ValueError(
                "The length of behavior_dist must be the same as the length of ranges."
            )
        elif len(identity_dist) != len(behavior_dist):
            raise ValueError(
                "The length of identity_dist must be the same as the length of behavior_dist."
            )

        self.__ranges = ranges
        self.__identity_dist = identity_dist
        self.__behavior_dist = behavior_dist
        self.__org_identity_dist = org_identity_dist

    def generate_user_trust(self):
        trust_score_df = self.__generate_user_trustscore()
        # trust_score_df["organization_trust"] = self.__org_identity_dist
        return trust_score_df

    def __generate_user_trustscore(self):
        user_id = range(0, self.user_count)
        identity_weights = [int(x * self.user_count) for x in self.__identity_dist]
        behavior_weights = [int(x * self.user_count) for x in self.__behavior_dist]
        org_identity_weights = [int(x * self.num_org) for x in self.__org_identity_dist]
        identity_trust = self.__generate_random_float(self.__ranges, identity_weights)
        behavior_trust = self.__generate_random_float(self.__ranges, behavior_weights)
        org_identity_trust = self.__generate_random_float(
            self.__ranges, org_identity_weights
        )
        affiliated_organization = [int(x // self.num_org) for x in user_id]
        data = {
            "user": user_id,
            "identity_trust": identity_trust,
            "behavior_trust": behavior_trust,
            "organization": affiliated_organization,
            "organization_trust": org_identity_trust,
        }
        print(
            f"Number of rows of user_id: {len(user_id)} identity_trust: {len(identity_trust)} behavior_trust: {len(behavior_trust)}"
        )
        return pd.DataFrame.from_dict(data)

    def __generate_random_float(self, ranges: list, weights: list):
        result = []
        i = 0
        while i < len(ranges):
            for n in range(weights[i]):
                random.seed(time.time())
                result.append(round(random.uniform(ranges[i][0], ranges[i][1]), 3))
            i += 1
        return result



import math
import random
from pathlib import Path
from pandas import DataFrame
from lorem_text import lorem
from .constants import DUA_COLS, PERMITTED_USE_OR_DISCLOSURE, DATA_CLASSES


class DUAGenerator:
    """
    Colums:
        - dataCustodian: data custodian id (not name)
        - recipient: data recipient id (not name)
        - requestdData: three categories from DATA_CLAASS in settings.py
        - permittedUseOrDisclosure: one categories from PERMITTED_USE_OR_DISCLOSURE in settings.py

        * Values of columns below are lorem ipsum for now *
        - terms
        - terminationCause
        - terminationEffect
        - dataSecurityPlanAccess
        - dataSecurityPlanProtection
        - dataSecurityPlanStorage

    TODO:
        - Refactor two methods
    """

    def __init__(self):
        # self.organization_df = None  # not used in the current version of the code.
        # self.rows = None  # not used in the current version of the code.
        self.num_organization = None
        self.num_dua_org = None
        # self.num_requested_data = 1
        self.seed = None
        self.main_dataclass = None
        self.main_dataclass_portion = None
        self.main_permitted_use_or_disclosure = None
        self.main_permitted_use_or_disclosure_portion = None
        self.save_path = None

    def generate_dua(self):
        """
        This method generates DUA records in the simplest way. Organizations generated by this
        methods does not match any organizations in the Synthea dataset.

        Parameters
        ----------
        num_organization: int
            Total number of organizations to generate.
        save_path: str
            Path to save the generated DUA records.
        num_dua_org: int
            Number of organizations who has DUA with the data custodian.
        num_requested_data: int
            Number of requested data categories.
        seed: int
            Random seed for data generation consistency.
        """

        num_org_main_dataclass = int(
            math.floor(self.num_dua_org * self.main_dataclass_portion / 100)
        )
        num_org_main_permitted_use_or_disclosure = int(
            math.floor(
                self.num_dua_org * self.main_permitted_use_or_disclosure_portion / 100
            )
        )

        output_string = (
            "Generating DUA...\n"
            + f"Number of organizations: {self.num_organization}\n"
            + f"Number of DUA organizations: {self.num_dua_org}\n"
            + f"Main data class: {self.main_dataclass}\n"
            + f"Main data class portion: {self.main_dataclass_portion}\n"
            + f"Number of main data class organizations: {num_org_main_dataclass}\n"
            + f"Main permitted use or disclosure: {self.main_permitted_use_or_disclosure}\n"
            + f"Main permitted use or disclosure portion: {self.main_permitted_use_or_disclosure_portion}\n"
            + f"Number of main permitted use or disclosure organizations: {num_org_main_permitted_use_or_disclosure}\n"
            + f"Random seed: {self.seed}\n"
        )
        print(output_string)

        random.seed(self.seed)
        data_custodian_id = "data_custodian"
        data_custodian_dummy = [data_custodian_id for i in range(self.num_dua_org)]
        data_recipients_id = random.sample(
            list(range(self.num_organization)), self.num_dua_org
        )
        permittedUseOrDisclosure = [
            random.choice(PERMITTED_USE_OR_DISCLOSURE) for i in range(self.num_dua_org)
        ]
        # requested_data = [
        #     self.__get_requested_data(self.num_requested_data)
        #     for i in range(self.num_dua_org)
        # ]
        requested_data = [random.choice(DATA_CLASSES) for i in range(self.num_dua_org)]
        main_dataclass_org_index = random.sample(
            range(self.num_dua_org), num_org_main_dataclass
        )
        for index in main_dataclass_org_index:
            requested_data[index] = self.main_dataclass
        main_usage_org_index = random.sample(
            range(self.num_dua_org), num_org_main_permitted_use_or_disclosure
        )
        for index in main_usage_org_index:
            permittedUseOrDisclosure[index] = self.main_permitted_use_or_disclosure

        term = [lorem.sentence() for i in range(self.num_dua_org)]
        terminationEffect = [lorem.sentence() for i in range(self.num_dua_org)]
        terminationCause = [lorem.sentence() for i in range(self.num_dua_org)]
        storage = [lorem.sentence() for i in range(self.num_dua_org)]
        access = [lorem.sentence() for i in range(self.num_dua_org)]
        protection = [lorem.sentence() for i in range(self.num_dua_org)]

        dua_df = DataFrame(
            list(
                zip(
                    data_custodian_dummy,
                    data_recipients_id,
                    requested_data,
                    permittedUseOrDisclosure,
                    term,
                    terminationCause,
                    terminationEffect,
                    access,
                    protection,
                    storage,
                )
            ),
            columns=DUA_COLS,
        )

        dua_df.to_csv(f"{self.save_path}/dua.csv")
        with open(f"{self.save_path}/detail.txt", "w") as f:
            f.write(output_string)

        return dua_df

    def __get_recipients(self):
        # First organization (index 0) is the data custodian.
        # Refer to `get_data_custodian` method belwo.
        organizations = self.organization_df["Id"].tolist()
        return organizations[1:]

    def __get_data_custodian(self):
        return self.organization_df["Id"].tolist()[0]

    def __get_requested_data(self, num_requested_data: int = 3):
        requested_data = random.sample(DATA_CLASSES, num_requested_data)
        return ",".join(requested_data)

    def generate_dua_from_synthea(
        self, organization_df: DataFrame, save_path: str, num_requested_data: int = 3
    ):
        """
        WARNING: This method is not used in the current version of the code.

        This method generate DUA records from the Synthea dataset. It assumes that organizatoins
        in the Synthea dataset exchange data with each other. Therefore, every organization in the
        organizations csv will have DUA records.
        """
        self.organization_df = organization_df
        self.rows = organization_df.shape[0]

        data_custodian = self.__get_data_custodian()
        data_custodian_dummy = [data_custodian for i in range(self.rows)]
        random.seed(self.random_seed)
        permittedUseOrDisclosure = [
            random.choice(PERMITTED_USE_OR_DISCLOSURE) for i in range(self.rows)
        ]
        recipient = self.__get_recipients()

        requested_data = [
            self.__get_requested_data(num_requested_data) for i in range(self.rows)
        ]

        term = [lorem.sentence() for i in range(self.rows)]
        terminationEffect = [lorem.sentence() for i in range(self.rows)]
        terminationCause = [lorem.sentence() for i in range(self.rows)]
        storage = [lorem.sentence() for i in range(self.rows)]
        access = [lorem.sentence() for i in range(self.rows)]
        protection = [lorem.sentence() for i in range(self.rows)]

        dua_df = DataFrame(
            list(
                zip(
                    data_custodian_dummy,
                    recipient,
                    requested_data,
                    permittedUseOrDisclosure,
                    term,
                    terminationCause,
                    terminationEffect,
                    access,
                    protection,
                    storage,
                )
            ),
            columns=DUA_COLS,
        )

        dua_df.to_csv(save_path)

        return dua_df



from rdflib import Namespace

SYNTHEA_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/synthea#"
DUA_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/data-usage-agreement#"
TRUST_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/trust#"
DATA_EXCHANGE_IRI = "https://knacc.umbc.edu/dae-young/kim/ontologies/data-exchange#"

TST = Namespace(TRUST_IRI)
DUA = Namespace(DUA_IRI)
SYN = Namespace(SYNTHEA_IRI)
DEX = Namespace(DATA_EXCHANGE_IRI)
